@@ -105,24 +105,32 @@ static int vv6450_set_size(struct quickcam *qc, unsigned int width, unsigned int
 {
 	int r = 0;
 	struct qc_sensor_data *sd = &qc->sensor_data;
-	if (qcdebug&QC_DEBUGLOGIC) PDEBUG("vv6450_set_size(quickcam=%p,width=%i,height=%i)",qc,width,height);
-
-	PDEBUG("vv6450_set_size(quickcam=%p,width=%i,height=%i)",qc,width,height);
-
+	//if (qcdebug&QC_DEBUGLOGIC)
+	  PDEBUG("vv6450_set_size(quickcam=%p,width=%i,height=%i)",qc,width,height);
 
 	if ( (width <= 162) && (height <= 124) )
 	{
 		sd->width = 162;
 		sd->height = 124;
+		STV_SET_CHECK(0x1505, 0x0f);
+		if(r) goto fail;
+	} else
+	if ( (width <= 176) && (height <= 144) )
+	{
+		/* exception for small QCIF even if not good fit */
+		sd->width = 162;
+		sd->height = 124;
+		qc->vwin.width = sd->width;
+		qc->vwin.height = sd->height;
 		PDEBUG("set %dx%d", sd->width, sd->height);
 		STV_SET_CHECK(0x1505, 0x0f);
 		if(r) goto fail;
+		return 0;
 	} else
 	if ( (width <= 162) && (height <= 248) )
 	{
 		sd->width = 162;
 		sd->height = 248;
-		PDEBUG("set %dx%d", sd->width, sd->height);
 		STV_SET_CHECK(0x1505, 0x08);
 		if(r) goto fail;
 	} else
@@ -130,7 +138,6 @@ static int vv6450_set_size(struct quickcam *qc, unsigned int width, unsigned int
 	{
 		sd->width = 324;
 		sd->height = 124;
-		PDEBUG("set %dx%d", sd->width, sd->height);
 		STV_SET_CHECK(0x1505, 0x04);
 		if(r) goto fail;
 	} else
@@ -138,9 +145,20 @@ static int vv6450_set_size(struct quickcam *qc, unsigned int width, unsigned int
 	{
 		sd->width = 324;
 		sd->height = 248;
+		STV_SET_CHECK(0x1505, 0x02);
+		if(r) goto fail;
+	} else
+	if ( (width <= 352) && (height <= 288) )
+	{
+		/* exception for CIF even if not good fit */
+		sd->width = 324;
+		sd->height = 248;
+		qc->vwin.width = sd->width;
+		qc->vwin.height = sd->height;
 		PDEBUG("set %dx%d", sd->width, sd->height);
 		STV_SET_CHECK(0x1505, 0x02);
 		if(r) goto fail;
+		return 0;
 	} else
 #ifdef TRY_HIGHRES
 	if ( (width <= 648) && (height <= 496) )
@@ -153,7 +171,6 @@ static int vv6450_set_size(struct quickcam *qc, unsigned int width, unsigned int
 	} else
 #endif
 	{
-	unknown:
 		PDEBUG("Unknown resolution %dx%d, set 324x248", width, height);
 		sd->width = 324;
 		sd->height = 248;
@@ -167,6 +184,9 @@ static int vv6450_set_size(struct quickcam *qc, unsigned int width, unsigned int
 
 		goto fail;
 	}
+	PDEBUG("set %dx%d", sd->width, sd->height);
+	qc->vwin.width = width;
+	qc->vwin.height = height;
 	return 0;
 fail:	return -1;
 }
@@ -208,13 +228,11 @@ fail:
 	return r;
 }
 
-#if COMPRESS
 struct stv_init {
 	const u8 *data;	/* If NULL, only single value to write, stored in len */
 	u16 start;
         u16 len;
 };
-#endif
 
 /* 
  * initialise parameters for vv6450 sensor.
@@ -355,8 +373,15 @@ static int vv6450_set_exposure(struct quickcam *qc, unsigned int val)
 	struct qc_sensor_data *sd = &qc->sensor_data;
 	int r = 0;
 	int a = (val >> 12) & 0x0F;
+#if 1
+	int oldval = sd->exposure;
+	if (val == oldval) return 0;
+	sd->exposure = val;
+	if ((oldval>>12) == a) return 0;
+#else
 	if((sd->exposure>>12) == a) return 0;
 	sd->exposure = val;
+#endif
 	if(a >= 14) a = 14;
 	if (qcdebug&QC_DEBUGADAPTATION) PDEBUG("vv6450_set_exposure %d (%d)",sd->exposure, a);
 
@@ -466,62 +491,42 @@ fail:
 	return r;
 }
 /* }}} */
-
-/* {{{ [fold] vv6450_set_target: Set target brightness for sensor autoexposure, val=0..65535 */
-static int vv6450_set_target(struct quickcam *qc, unsigned int val)
-{
-	//struct qc_sensor_data *sd = &qc->sensor_data;
-	int r = 0;
-
-	if (val == qc->vpic.brightness) return 0;
-	qc->vpic.brightness = val;
-
-	PDEBUG("vv6450_set_target: val=%d", val);
-#if 0
-        r = qc_i2c_wait(qc);
-	if(r) goto fail;
-	STV_SET_CHECK(0x1501, val & 0xff);
-        r = qc_i2c_wait(qc);
-	if(r) goto fail;
-	STV_SET_CHECK(0x1502, (val >> 8) & 0xff);
-        r = qc_i2c_wait(qc);
-	if(r) goto fail;
-	STV_SET_CHECK(0x1503, (val >> 16) & 0xff);
-	r = qc_i2c_wait(qc);
-	if(r) goto fail;
-	STV_SET_CHECK(0x143F, 0x01);
-#endif
-	if(r) goto fail;
-	return r;
-fail:
-	PDEBUG("error set_target");
-	return r;
-}
-/* }}} */
-
-
 /* {{{ [fold] vv6450_set_shutter: Set shutter value for sensor, val=0..65535 */
 static int vv6450_set_shutter(struct quickcam *qc, unsigned int val)
 {
 	struct qc_sensor_data *sd = &qc->sensor_data;
 	int r;
-
-	if ((val>>7) == (sd->shutter>>7)) return 0;
-	sd->shutter = val;
+	int oldval;
 
 	//PDEBUG("vv6450_set_shutter: val=%d", val);
 
+	if (val == sd->shutter) return 0;
+	oldval = sd->shutter;
+	sd->shutter = val;
+#define SHUTTER_SHIFT 8
+
+	if ((val>>SHUTTER_SHIFT) == (oldval>>SHUTTER_SHIFT)) return 0;
+	
+	/* rescale the value if greater than 0xC000 */
+	if(val > 0xC000) {
+	  val = 0xC000 + (val&0x3FFF)*8;
+	}
 	/*
-	  0xFFFF -> 01 FF
-	  0x8000 -> 01 00
-	  0x0000 -> 00 00
+	  0xZZZZ -> 03 FF  (max value, but never used)
+	  0xFFFF -> 02 B0  (extreme light and slow frame rate)
+	  0xC100 -> 00 C8
+	  0xC000 -> 00 C0
+	  0x8000 -> 00 80  (normal value)
+	  0x0000 -> 00 00  (extreme dark and fast frame rate)
 	*/
+	//PDEBUG("vv6450_set_shutter: val=%d (%04X)", val, (val >> SHUTTER_SHIFT));
+
         r = qc_i2c_wait(qc);
 	if(r) goto fail;
-	STV_SET_CHECK(0x143d, (val >> 7) & 0xff);
+	STV_SET_CHECK(0x143d, (val >> SHUTTER_SHIFT) & 0xff);
         r = qc_i2c_wait(qc);
 	if(r) goto fail;
-	STV_SET_CHECK(0x143e, (val >> 15) & 0xff);
+	STV_SET_CHECK(0x143e, (val >> (SHUTTER_SHIFT+8)) & 0x03);
 	r = qc_i2c_wait(qc);
 	if(r) goto fail;
 	STV_SET_CHECK(0x143F, 0x01);
@@ -531,6 +536,33 @@ fail:
 	return r;
 }
 /* }}} */
+/* {{{ [fold] vv6450_set_target: Set target brightness for sensor autoexposure, val=0..65535 */
+static int vv6450_set_target(struct quickcam *qc, unsigned int val)
+{
+	int r = 0;
+
+	//PDEBUG("vv6450_set_target: val=%d", val);
+
+#ifdef WHITENESS_IS_SHUTTER
+	if (val == qc->vpic.whiteness) return 0;
+	qc->vpic.whiteness = val;
+#else
+	if (val == qc->vpic.brightness) return 0;
+	qc->vpic.brightness = val;
+#endif
+
+	//PDEBUG("vv6450_set_target: call set_shutter (%d) %d", val, qc->settings.shutteradapt);
+
+	if(!qc->settings.shutteradapt) {
+	  r = vv6450_set_shutter(qc, val);
+	}
+	if(r) goto fail;
+	return r;
+fail:
+	PDEBUG("error set_target");
+	return r;
+}
+/* }}} */
 
 const struct qc_sensor qc_sensor_vv6450 = {
 	name:		"VV6450",
