@@ -1,6 +1,14 @@
 #ifndef _LINUX_QUICKCAM_H
 #define _LINUX_QUICKCAM_H
 
+/*
+ * This define change behaviour of the whiteness control.
+ *  If defined, whiteness controll will set shutter-value instead.
+ *  This should only be used if module is loaded with shutteradapt=0
+ *  otherwise shutter-value will be changed at once.
+ */
+#define WHITENESS_IS_SHUTTER
+
 /* {{{ [fold] SECTION: common definitions with userspace applications */
 #define QC_LUT_SIZE	(3*256)		/* Lookup table definition for equalization */
 #define QC_LUT_RED	0
@@ -47,6 +55,10 @@ struct qc_userlut {
 #define VIDIOCQCSVIDEONR	_IOWR('v',QC_IOCTLBASE+11,int)	/* Set videodevice number (/dev/videoX) */
 #define VIDIOCQCGUSERLUT	_IOR ('v',QC_IOCTLBASE+12,struct qc_userlut)	/* Get user-specified lookup-table correction */
 #define VIDIOCQCSUSERLUT	_IOWR('v',QC_IOCTLBASE+12,struct qc_userlut)	/* Set user-specified lookup-table correction */
+#define VIDIOCQCGSHUTTERADAPT	_IOR ('v',QC_IOCTLBASE+13, int)	/* Get automatic adaptive shutter control (0-1) */
+#define VIDIOCQCSSHUTTERADAPT	_IOWR('v',QC_IOCTLBASE+13, int)	/* Set automatic adaptive shutter control (0-1) */
+#define VIDIOCQCGSHUTTERVAL	_IOR ('v',QC_IOCTLBASE+14, int)	/* Get automatic adaptive shutter control value */
+#define VIDIOCQCSSHUTTERVAL	_IOWR('v',QC_IOCTLBASE+14, int)	/* Set automatic adaptive shutter control (0-65535) */
 
 #define VIDIOCQCGSTV		_IOWR('v',QC_IOCTLBASE+20,int)	/* Read STV chip register */
 #define VIDIOCQCSSTV		_IOW ('v',QC_IOCTLBASE+20,int)	/* Write STV chip register */
@@ -98,15 +110,16 @@ struct qc_userlut {
 #include <linux/spinlock.h>
 
 /* {{{ [fold] SECTION: user configuration */
-#define VERSION		"QuickCam USB $Date: 2004/07/29 18:12:39 $"
+#define VERSION		"QuickCam Messenger/Communicate USB $Date: 2004/12/21 10:00:00 $"
 #ifndef COMPRESS
-#define COMPRESS	1		/* 1=include compression support, 0=otherwise */
+#define COMPRESS	0		/* 1=include compression support, 0=otherwise */
 #endif
 #ifndef DEBUGLEVEL
 //#define DEBUGLEVEL	0
 //#define DEBUGLEVEL	(QC_DEBUGERRORS|QC_DEBUGADAPTATION)
 //#define DEBUGLEVEL	(QC_DEBUGERRORS|QC_DEBUGUSER)
 #define DEBUGLEVEL	(QC_DEBUGERRORS)
+//#define DEBUGLEVEL	(0xFFFF)
 #endif
 #ifdef NDEBUG				/* Enable debugging if DEBUG is defined; if (also) NDEBUG is defined, disable debugging */
 #undef DEBUG
@@ -490,7 +503,7 @@ struct qc_settings_data {
 	unsigned int frameskip     : 4;	/* How many frames to skip (higher=lower framerate) */
 	unsigned int quality       : 3;	/* Quality of format conversion (higher=better but slower) */
 	unsigned int adaptive      : 1;	/* Use automatic exposure control */
-	unsigned int shutter_adapt : 1;	/* Use automatic shutter control */
+	unsigned int shutteradapt  : 1;	/* Use automatic shutter control */
 	unsigned int equalize      : 1;	/* Equalize images */
 	unsigned int userlut       : 1;	/* Apply user-specified lookup-table */
 	unsigned int retryerrors   : 1;	/* If errors happen when capturing an image, retry a few times? */
